/* ser.c
	(C) 2004-5 Captain http://www.captain.at
	
	Sends 3 characters (ABC) via the serial port (/dev/ttyS0) and reads
	them back if they are returned from the PIC.
	
	Used for testing the PIC-MMC test-board
	http://www.captain.at/electronic-index.php

*/
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

#include "captain/capser.h"


//#define BITRATE B9600
//#define BITRATE B19200
//#define BITRATE B38400
#define BITRATE B115200

#define FILENAME "Blinker1.eeprom"
#define FILESIZE 0x8000
//0x8000

#define COMDEV "/dev/com6"

int fd;

unsigned char _LFSR;

int initport(int fd) {
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 19200...
	cfsetispeed(&options, BITRATE);
	cfsetospeed(&options, BITRATE);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);
	return 1;
}


int IterateLFSR()
        {
                int result = (_LFSR & 1) == 1;
                _LFSR = (unsigned char)((_LFSR << 1) | (_LFSR >> 7 ^ _LFSR >> 5 ^ _LFSR >> 4 ^ _LFSR >> 1) & 1);
                return result;
        }


void convertToEncoded(unsigned int data, unsigned char* buf){

//unsigned char buf[11]; 
 
//encode command
int i;
for (i = 0; i < 11; ++i)
            {  
                buf[i] = (unsigned char)(0x92 | ((unsigned char)(i == 10 ? 255 : 0) & 0x60) | (data & 1) | (data & 2) << 2 | (data & 4) << 4);
                data >>= 3;  
            }//for  


/*
for(i=0;i<32;i++){
 buf[i] = (0x1 & data) ? 0xff : 0xfe;
 data = data >> 1;
}
 */

//printf("transmitting buffer: \n");
//for(i=0;i<11;i++)
// printf ("  %x\n", buf[i]);
 
//send encoded command 
/*        if (!writeport(fd, buf, 11)) {
                printf("write failed\n");
                close(fd);
                return 1;
	}//if
*/
}

int main(int argc, char **argv) {


//open binary
//buffer size: 1 + 250 + 258
unsigned char lfsrbuf[509];
//bin buffer size: 1 + 1 + 11*(0x8000+3)/4 = 90114
unsigned char binbuf[90114]  ;
int i;

//init
_LFSR = 'P';//0x50;

// read in the binary file into buffer
FILE* binf = fopen(FILENAME, "rb");
if(!binf){ 
 printf("file not found\n");
 return -1; 
} 

for(i=0;i<FILESIZE;i++){ 
 binbuf[i] = getc(binf);
//printf("data read: %x, ",binbuf[i]); 
 
} 
printf("\n");

fclose(binf);



lfsrbuf[0]=0xf9;
// set up lfsr's
for (i = 1; i < 251; ++i)
                lfsrbuf[i] = (unsigned char)(0xFE | (IterateLFSR() ? 1 : 0));

// set up f9's
for (; i < 258+251; ++i)
                lfsrbuf[i] = 0xF9;


//prepare the command, progsize and program buffer

unsigned int cmd = 1; //1 is command to write into RAM. use 3 for writting into EEPROM
 convertToEncoded((unsigned int)cmd, &(binbuf[0]));

//send program size
           int longCount = (FILESIZE + 3) >> 2;
            convertToEncoded((unsigned int)longCount, &(binbuf[1]));
 
//send program
           for (i = 0; i < longCount; ++i)
            {
                int byteIdx = (i << 2);
                unsigned int x = (unsigned int)(
                    (binbuf[byteIdx + 3] << 24) |
                    (binbuf[byteIdx + 2] << 16) |
                    (binbuf[byteIdx + 1] << 8) |
                    (binbuf[byteIdx]));
//printf("submitting #%d: %x, %x, %x, %x\n", i, (unsigned char)(x>>0), (unsigned char)(x>>8), (unsigned char)(x>>16),(unsigned char)(x>>24));
 
convertToEncoded((unsigned int)x, &(binbuf[2+i*11]));

            }








//open serial comm
 fd = open(COMDEV, O_RDWR | O_NOCTTY | O_NDELAY);
        if (fd == -1) {
                perror("open_port: Unable to open /dev/ttyS0 - ");
                return 1;
        } else { 
                fcntl(fd, F_SETFL, 0);
        } 
 
// simulate reset (DTR)
close(fd);
usleep(10000); 
fd = open(COMDEV, O_RDWR | O_NOCTTY | O_NDELAY);
fcntl(fd, F_SETFL, 0);
 
//        printf("baud=%d\n", getbaud(fd));
        initport(fd);
//        printf("baud=%d\n", getbaud(fd));


//delay for 100ms
usleep(100000);


// send lfsr stuff

        if (!writeport(fd, lfsrbuf, 509)) {
                printf("write failed\n");
                close(fd);
                return 1;
        }

//flush(fd);


//printf("check\n");

//tcflush(fd, TCOFLUSH);

// receive lfsr's
        usleep(70000); //THIS IS CRITICAL, 5000 is too small, 200000 is too large. 10000 and 50000 works 
        unsigned char sResult[250]; 
//        fcntl(fd, F_SETFL, FNDELAY); // don't block serial read
 
        if (!readport(fd,sResult, 250)) {
                printf("read failed\n");
                close(fd); 
                return 1; 
        } 
        //printf("readport=%s\n", sResult);

//printf("check 2\n");
// test
int test = 0;
int curlfsr;
for(i=0;i<250;i++){
 curlfsr=IterateLFSR();
// printf("comparing %d and %d\n", (sResult[i]-0xfe), curlfsr);
 if((sResult[i]-0xfe) != curlfsr/*IterateLFSR()*/){
  test=1;
  break;
 }
}
if(test){
 printf("test FAILED on %d\n", i);
 return -1;
}


//receive version

        if (!readport(fd,sResult, 8)) {
                printf("read failed\n");
                close(fd);
                return 1;
        }  

unsigned char version = 0;

            for (i = 0; i < 8; ++i)
            {
                version >>= 1;
                version += (unsigned char)((sResult[i]-0xfe) ? 128 : 0);
            }

printf("detected version: %d\n", (int)version);


//send cmd, progsize, prog
if (!writeport(fd, binbuf, 90114)) {
                printf("write failed\n"); 
                close(fd); 
                return 1; 
        }//if 


unsigned char res;
unsigned char f9 = 0xf9;

writeport(fd, &f9, 1);

//flush
tcflush(fd, TCOFLUSH);
 
//usleep(250000);

fcntl(fd, F_SETFL, FNDELAY); // don't block serial read
        while (!readport(fd,&res,1)) {
		usleep(100); //10ms
                writeport(fd, &f9, 1);
	//	printf("ping\n");
                
                
        }

printf("received %d\n",(unsigned char)(res-0xfe));


// read checksum

while(1){
        usleep(500000);
}


	
	usleep(500000);
	//char sResult[254];
//	fcntl(fd, F_SETFL, FNDELAY); // don't block serial read

	if (!readport(fd,sResult,250)) {
		printf("read failed\n");
		close(fd);
		return 1;
	}
	printf("readport=%s\n", sResult);



	close(fd);
	return 0;

}//main


